@@ -6,14 +6,12 @@ exceptions used for message-passing" to simply "we needed to express an error
 condition in a way easily told apart from other, truly unexpected errors".
 """
 
-from collections import namedtuple
 from traceback import format_exception
 from pprint import pformat
 
-try:
-    from .vendor import six
-except ImportError:
-    import six
+from .util import six
+
+from .util import encode_output
 
 
 class CollectionNotFound(Exception):
@@ -72,15 +70,23 @@ class UnexpectedExit(Failure):
         if 'stdout' not in self.result.hide:
             stdout = already_printed
         else:
-            stdout = _tail(self.result.stdout)
+            stdout = encode_output(
+                _tail(self.result.stdout),
+                self.result.encoding,
+            )
         if self.result.pty:
             stderr = " n/a (PTYs have no stderr)"
         else:
             if 'stderr' not in self.result.hide:
                 stderr = already_printed
             else:
-                stderr = _tail(self.result.stderr)
-        return """Encountered a bad command exit code!
+                stderr = encode_output(
+                    _tail(self.result.stderr),
+                    self.result.encoding,
+                )
+        command = self.result.command
+        exited = self.result.exited
+        template = """Encountered a bad command exit code!
 
 Command: {0!r}
 
@@ -90,7 +96,8 @@ Stdout:{2}
 
 Stderr:{3}
 
-""".format(self.result.command, self.result.exited, stdout, stderr)
+"""
+        return template.format(command, exited, stdout, stderr)
 
     def __repr__(self):
         # TODO: expand?
@@ -177,15 +184,6 @@ class UnknownFileType(Exception):
     pass
 
 
-#: A namedtuple wrapping a thread-borne exception & that thread's arguments.
-#: Mostly used as an intermediate between `.ExceptionHandlingThread` (which
-#: preserves initial exceptions) and `.ThreadException` (which holds 1..N such
-#: exceptions, as typically multiple threads are involved.)
-ExceptionWrapper = namedtuple(
-    'ExceptionWrapper',
-    'kwargs type value traceback'
-)
-
 def _printable_kwargs(kwargs):
     """
     Return print-friendly version of a thread-related ``kwargs`` dict.
@@ -218,10 +216,10 @@ class ThreadException(Exception):
         Threads which did not encounter an exception, do not contribute to this
         exception object and thus are not present inside `exceptions`.
     """
-    #: A tuple of `ExceptionWrappers <ExceptionWrapper>` containing the initial
-    #: thread constructor kwargs (because `threading.Thread` subclasses should
-    #: always be called with kwargs) and the caught exception for that thread
-    #: as seen by `sys.exc_info` (so: type, value, traceback).
+    #: A tuple of `ExceptionWrappers <invoke.util.ExceptionWrapper>` containing
+    #: the initial thread constructor kwargs (because `threading.Thread`
+    #: subclasses should always be called with kwargs) and the caught exception
+    #: for that thread as seen by `sys.exc_info` (so: type, value, traceback).
     #:
     #: .. note::
     #:     The ordering of this attribute is not well-defined.
